// This code is for reference only. It was written as a hobby by someone who is not a professional programmer.
// The author accepts nor liability for this code or any damage it may cause if used. No support will be provided.


#include "sr2_keymap.h"

const int DataPin = 23;
const int IRQpin =  7;
int led = 11;


// INTERRUPT

volatile int current_bit = 0;
volatile int start_bit = 0;
volatile uint8_t scancode = 0b00000000;
volatile int parity_bit = 0;
volatile int stop_bit = 0;

void ps2interrupt(void)
{
  int data;
  data = digitalRead(DataPin);
//  Serial.print(data);
  if (current_bit == 0) {
    start_bit = data;
  } else if (current_bit > 0 && current_bit < 9) {
    int scancode_bit = current_bit - 1;
    bitWrite(scancode, scancode_bit, data);
  } else if (current_bit == 9) {
    parity_bit = data;
  } else if (current_bit == 10) {
    stop_bit = data;
  }
  current_bit++;
}
bool verify_input() {
  bool verified = false;
  if (start_bit == 0 && stop_bit == 1) {
    bool parity_check = false;
    for (int i = 0; i < 8; i++) {
      if (bitRead(scancode, i) == 1) {
        parity_check = !parity_check;
      }
    }
    if (parity_bit == 1) {
      parity_check = !parity_check;
    }
    if (parity_check == true) {
      verified = true;
    }
    else {
      Serial.println("Parity check failed");
    }
  }
  else {
    Serial.println("Start / stop bit check failed");
    Serial.println(current_bit);
    Serial.println(start_bit);
    Serial.println(scancode);
    Serial.println(parity_bit);
    Serial.println(stop_bit);
  }
  return verified;
}

// 1 2 3 A  HE
// 4 5 6 B  LP
// 7 8 9 C  ST
// * 0 # D  OP

// Helper functions
#define NUMELEMENTS(x) (sizeof(x) / sizeof(x[0]))

int ctrl_state = 0;
int shift_state = 0;
int alt_state = 0;
int win_state = 0;

struct HOTKEY
{
  int ctrl;
  int shift;
  int alt;
  int win;
  unsigned int key;
};

HOTKEY hotkey_1 = {1, 1, 1, 0, KEY_F1};
HOTKEY hotkey_2 = {1, 1, 1, 0, KEY_F2};
HOTKEY hotkey_3 = {1, 1, 1, 0, KEY_F3};
HOTKEY hotkey_A = {1, 1, 1, 0, KEY_A};

HOTKEY hotkey_4 = {1, 1, 1, 0, KEY_F4};
HOTKEY hotkey_5 = {1, 1, 1, 0, KEY_F5};
HOTKEY hotkey_6 = {1, 1, 1, 0, KEY_F6};
HOTKEY hotkey_B = {1, 1, 1, 0, KEY_B};

HOTKEY hotkey_7 = {1, 1, 1, 0, KEY_F7};
HOTKEY hotkey_8 = {1, 1, 1, 0, KEY_F8};
HOTKEY hotkey_9 = {1, 1, 1, 0, KEY_F9};
HOTKEY hotkey_C = {1, 1, 1, 0, KEY_C};

HOTKEY hotkey_ASTERISK  = {1, 1, 1, 0, KEY_8};
HOTKEY hotkey_0         = {1, 1, 1, 0, KEY_F10};
HOTKEY hotkey_OCTOTHORP = {1, 1, 1, 0, KEY_3};
HOTKEY hotkey_D         = {1, 1, 1, 0, KEY_D};

HOTKEY hotkey_HELP = {1, 1, 1, 0, KEY_H};
HOTKEY hotkey_STOP = {1, 1, 1, 0, KEY_S};

struct HOTKEYS
{
  int dn_scancode;
  int up_scancode;
  HOTKEY hotkey;
  int numElements;
};

HOTKEYS hotkeyMap[] =
{
  {DN_1, UP_1, hotkey_1},
  {DN_2, UP_2, hotkey_2},
  {DN_3, UP_3, hotkey_3},
  {DN_A, UP_A, hotkey_A},
  {DN_4, UP_4, hotkey_4},
  {DN_5, UP_5, hotkey_5},
  {DN_6, UP_6, hotkey_6},
  {DN_B, UP_B, hotkey_B},
  {DN_7, UP_7, hotkey_7},
  {DN_8, UP_8, hotkey_8},
  {DN_9, UP_9, hotkey_9},
  {DN_C, UP_C, hotkey_C},
  {DN_ASTERISK, UP_ASTERISK, hotkey_ASTERISK},
  {DN_0, UP_0, hotkey_0},
  {DN_OCTOTHORP, UP_OCTOTHORP, hotkey_OCTOTHORP},
  {DN_D, UP_D, hotkey_D},
  {DN_HELP, UP_HELP, hotkey_HELP},
  {DN_STOP, UP_STOP, hotkey_STOP},
};

void setup() {
  delay(1000);
  //  Serial.begin(9600);
  Serial.println("Starting...");
  Keyboard.releaseAll();
  pinMode(led, OUTPUT);
  // Flash LED on start up
  int led_flash_delay = 50;
  for (int i = 0; i < 4; i++) {
    digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
    delay(led_flash_delay);               // wait for a second
    digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
    delay(led_flash_delay);
  }

  // Set up keyboard pins
  pinMode(DataPin, INPUT_PULLUP);
  pinMode(IRQpin, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(IRQpin), ps2interrupt, FALLING);
  current_bit = 0;
  scancode = 0b00000000;
  parity_bit = 0;
  stop_bit = 0;
  Serial.println("All variables reset. Ready.");
}


void loop() {
  if (current_bit > 10) {
    if (verify_input() == true) {
      
//      Serial.print(ctrl_state);
//      Serial.print(shift_state);
//      Serial.print(alt_state);
//      Serial.println(scancode);
      for (int i = 0; i < NUMELEMENTS(hotkeyMap); i++)
      {
        if (scancode == hotkeyMap[i].dn_scancode || hotkeyMap[i].up_scancode == scancode) {
          if (hotkeyMap[i].dn_scancode == scancode) {
            ctrl_state += hotkeyMap[i].hotkey.ctrl;
            shift_state += hotkeyMap[i].hotkey.shift;
            alt_state += hotkeyMap[i].hotkey.alt;
            win_state += hotkeyMap[i].hotkey.win;
          }

          if (hotkeyMap[i].up_scancode == scancode) {
            ctrl_state -= hotkeyMap[i].hotkey.ctrl;
            shift_state -= hotkeyMap[i].hotkey.shift;
            alt_state -= hotkeyMap[i].hotkey.alt;
            win_state -= hotkeyMap[i].hotkey.win;
          }
          if (ctrl_state > 0) {
            Keyboard.press(MODIFIERKEY_CTRL);
          } else {
            Keyboard.release(MODIFIERKEY_CTRL);
          }
          if (shift_state > 0) {
            Keyboard.press(MODIFIERKEY_SHIFT);
          } else {
            Keyboard.release(MODIFIERKEY_SHIFT);
          }
          if (alt_state > 0) {
            Keyboard.press(MODIFIERKEY_ALT);
          } else {
            Keyboard.release(MODIFIERKEY_ALT);
          }
          if (win_state > 0) {
            Keyboard.press(MODIFIERKEY_GUI);
          } else {
            Keyboard.release(MODIFIERKEY_GUI);
          }
          if (hotkeyMap[i].dn_scancode == scancode) {
            Keyboard.press(hotkeyMap[i].hotkey.key);
          }
          if (hotkeyMap[i].up_scancode == scancode) {
            Keyboard.release(hotkeyMap[i].hotkey.key);
          }
        }
      }
    } else {
      //      Serial.println("Unverified scancode!");
      Keyboard.releaseAll();
      ctrl_state = 0;
      shift_state = 0;
      alt_state = 0;
      win_state = 0;
    }
    current_bit = 0;
    scancode = 0b00000000;
    parity_bit = 0;
    stop_bit = 0;
  }
}
